var CDSWebVisuAccess, IecTypes, TypeIds, PromiseCallbacks, promiseDict = {}, iPromiseId = 0;

/**
 * Interface between window and iFrame of a Native Element (HTML5 control).
 */
window.addEventListener("message", function(message) {
	if (message.origin !== window["CdsInfo"]["TargetOrigin"]) {
		return;
	}
	if (message.data === null || message.data === undefined) {
		return;
	}
	console.log(message.data);
	var promiseId = message.data.data.promiseId, promiseCallbacksHelp = null;
	if (promiseId !== undefined) {
		promiseCallbacksHelp = promiseDict[promiseId];
		if (promiseCallbacksHelp === undefined) {
			promiseCallbacksHelp = null;
		} else {
			delete promiseDict[promiseId];
		}
	}

	if (message.data.type === "ErrorMessage") {	
		if (promiseCallbacksHelp !== null)
		{
			promiseCallbacksHelp.getRejectCallback()(message.data.data.error);
		}
	}
	else if (message.data.type === "MethodCall") {
		var methodName = message.data.data.methodName;		
		var methodNameComponents = methodName.split(".")
			, method = window["CdsInfo"]["Wrapper"];
		for (var i = 0; i < methodNameComponents.length - 1; ++i)
			method = method[methodNameComponents[i]]();
		method[methodNameComponents[methodNameComponents.length - 1]].apply(method, message.data.data.params);
	}
	else if (message.data.type === "CheckSimpleValueResult" || message.data.type === "SetSimpleValueResult") {
		if (promiseCallbacksHelp !== null)
		{
			if (message.data.data.result)
				promiseCallbacksHelp.getResolveCallback()(message.data.data.value);
			else
				promiseCallbacksHelp.getRejectCallback()(message.data.data.value);
		}
	}
	else if (message.data.type === "GetTypeDescResult") {
		if (promiseCallbacksHelp !== null)
		{
			if (message.data.data.result)
				promiseCallbacksHelp.getResolveCallback()(message.data.data.typeDesc);
			else
				promiseCallbacksHelp.getRejectCallback()(message.data.data.typeDesc);
		}
	}
	else if (message.data.type === "Resize") {
		var docBody = document.body;
		docBody.style.width = message.data.data.width+'px';
		docBody.style.height = message.data.data.height+'px';
	} else if (message.data.type === "GetAdditionalFileNameResult") {
		if (promiseCallbacksHelp !== null)
		{
			if (message.data.data.result)
				promiseCallbacksHelp.getResolveCallback()(message.data.data.resultingAdditionalFileName);
			else
				promiseCallbacksHelp.getRejectCallback()(message.data.data.resultingAdditionalFileName);
		}
	} else if (message.data.type === "GetImagePoolFileNameResult") {
		if (promiseCallbacksHelp !== null)
		{
			if (message.data.data.result)
				promiseCallbacksHelp.getResolveCallback()(message.data.data.resultingFileName);
			else
				promiseCallbacksHelp.getRejectCallback()(message.data.data.resultingFileName);
		}
	}
	else if (message.data.type === "CheckComplexValueResult" || message.data.type === "SetComplexValueResult") {
		if (promiseCallbacksHelp !== null)
		{
			if (message.data.data.result)
				promiseCallbacksHelp.getResolveCallback()(message.data.data.value, message.data.data.indexCount, message.data.data.index0, message.data.data.index1, message.data.data.index2);
			else
				promiseCallbacksHelp.getRejectCallback()(message.data.data.value, message.data.data.indexCount, message.data.data.index0, message.data.data.index1, message.data.data.index2);
		}
	}
});

(function () {
	/** enum {number} with all scalar type classes 
	* @memberof Constants#
	* @readonly
	* @enum {number}
	*/
	IecTypes = {
		/** 0 */
		Bool:		0,
		/** 1 */
		Bit:		1,
		/** 2 */
		Byte:		2,
		/** 3 */
		Word:		3,
		/** 4 */
		DWord:		4,
		/** 5 */
		LWord:		5,
		/** 6 */
		SInt:		6,
		/** 7 */
		Int:		7,
		/** 8 */
		DInt:		8,
		/** 9 */
		LInt:		9,
		/** 10 */
		USInt:		10,
		/** 11 */
		UInt:		11,
		/** 12 */
		UDInt:		12,
		/** 13 */
		ULInt:		13,
		/** 14 */
		Real:		14,
		/** 15 */
		LReal:		15,
		/** 16 */
		String:	16,
		/** 17 */
		WString:	17,
		/** 18 */
		Time:		18,
		/** 19 */
		Date:		19,
		/** 20 */
		DateAndTime:	20,
		/** 21 */
		TimeOfDay:		21
	};
	
	/** enum {number} with all specific type ids 
	* @memberof Constants#
	* @readonly
	* @enum {number}
	*/
	TypeIds = {
		/** 997 */
		Color:		997,
		/** 998 */
		Font:		998
	};
}());


(function()
{
	
	/**
	 * Provides functions that HTML5 elements can call to
	 * interact with IEC code.
	 */
	CDSWebVisuAccess = function (){
	};
	
	CDSWebVisuAccess.prototype = {

		/**
		 * Sends a scalar value to IEC code.
		 * @param {String} methodName The method name as configured in the column "Call method" in the HTML5 editor.
		 * @param {*} param The scalar value that will be sent to the IEC application.
		 * @returns {Promise<value>} A promise that is resolved or rejected.
		 */
		sendSimpleValue: function(methodName, param) {
			return this._postMessage("SetValue", {methodName: methodName, param:param}, true);
		},
		
		/**
		 * Checks if a scalar value can be written into the configured variable.
		 * @param {String} methodName The method name as configured in the column "Call method" in the HTML5 editor.
		 * @param {*} param The scalar value that will be sent to the IEC application. Can be number, bigint, date or another type.
		 * @returns {Promise<value>} Promise.Resolve() is called if the value can be written into the specified variable. Otherwise, the promise is rejected.
		 */
		checkSimpleValue: function(methodName, param) {
			return this._postMessage("CheckValue",{methodName: methodName, param:param}, true);
		},
		
		/**
		 * Sends a mouse event for this Html element to the IEC application.
		 * If the element has input actions like 'OnMouseClick' or 'OnMouseDown' configured, these events can be triggered here.
		 * @param {MouseEvent} event The mouse event coming from Javascript code. Event must be of type [MouseEvent]{@link https://developer.mozilla.org/en-US/docs/Web/API/MouseEvent}.
		 * @returns {Promise} A resolved promise. Not relevant.
		 */
		sendMouseEvent: function(event) {
			if (!(event instanceof MouseEvent))
				throw new TypeError("Expected MouseEvent object");
			return this._postMessage("SendMouseEvent",{type: event.type, xPos: event.x, yPos: event.y}, false);
		},
		
		/**
		 * Returns the type description for the corresponding type Id.
		 * Typically the method call of an element has the following signature: function(value, type, typeid) {...}
		 * The value is the value itself and the typeid identifies the type description which provides further information about
		 * the value like size or dimensions in case of an array type.
		 * 
		 * @param {Number} typeId The type Id that comes from the element's method call together with the value.
		 * @returns {Promise<typeDesc>} A promise that contains a JSON object with the members descripted in the type descriptions section starting with TypeDesc..
		 */
		getTypeDesc: function(typeId) {
			return this._postMessage("GetTypeDesc", {typeId: typeId}, true);
		},

		/**
		 * Gets the actual filename of a additional file that can be referenced in the element.
		 * The original filenames are renamed after downloading the application in order to prevent name collisions with other elements.
		 * @param {String} additionalFile The original name of the file specified in the 'Additional Files' section in the Html5 Editor.
		 * @returns {Promise<resultingAdditionalFileName>} A promise that contains the resulting file name.
		 */
		getAdditionalFile: function(additionalFile) {
			return this._postMessage("GetAdditionalFileName", {originalAdditionalFileName: additionalFile}, true);
		},

		/**
		 * Gets the actual filename of a provided image file that can be referenced in the element.
		 * The original filenames are renamed after downloading the application in order to prevent name collisions with other elements.
		 * @param {String} imageId The original name of the file specified in the 'Image' section in the Html5 Editor.
		 * @returns {Promise<resultingFileName>} A promise that contains the resulting file name.
		 */
		getImagePoolFileName: function(imageId) {
			return this._postMessage("GetImagePoolFileName", {imageId: imageId}, true);
		},
		
		/**
		 * Notifies the IEC application about which parts of the array values should be sent.
		 * Can be used for example with a scrollable table element that only receives the values that are currently visible.
		 * New values may be requested after the user has scrolled.
		 * @param {String} methodName The method name as configured in the column "Call method" in the HTML5 editor.
		 * @param {Number} startIndex Specifies the zero-based start of the array range.
		 * @param {Number} endIndex Specifies the zero-based end of the array range.
		 * @param {Number} scrollDimension The dimension of the array values that should be retrieved.
		 * @returns {Promise} A resolved promise. Not relevant.
		 */
		setScrollRange: function(methodName, startIndex, endIndex, scrollDimension) {
			return this._postMessage("SetScrollRange", {methodName: methodName, startIndex:startIndex, endIndex:endIndex, scrollDimension:scrollDimension}, false);
		},
		
		/**
		 * Checks if a certain value of a complex value (array-like structure) can be written to into the configured variable.
		 * @param {String} methodName The method name as configured in the column "Call method" in the HTML5 editor.
		 * @param {*} param The scalar value that will be sent to the IEC application. Can be of the types number, bigint, date or others.
		 * @param {Number} indexCount Number of dimensions of the configured datatype. E.g. 2 for a simple table. Maximum is 3.
		 * @param {Number} index0 Zero-Based index in the first dimension.
		 * @param {Number} index1 Zero-Based index in the second dimension.
		 * @param {Number} index2 Zero-Based index in the third dimension.
		 * @returns {Promise<value>} Resolve() is called if the value can be written into the specified variable. Otherwise, the promise is rejected.
		 */
		checkComplexValue: function(methodName, param, indexCount, index0, index1, index2) {  // 
			return this._postMessage("CheckComplexValue", {methodName: methodName, param:param, indexCount:indexCount, index0:index0, index1:index1, index2:index2}, true);
		},
		
		/**
		 * Sends a certain value of a complex value (array-like structure) to the configured variable in the application.
		 * @param {String} methodName The method name as configured in the column "Call method" in the HTML5 editor.
		 * @param {*} param The scalar value that will be sent to the IEC application.
		 * @param {Number} indexCount Number of dimensions of the configured datatype. E.g. 2 for a simple table. Maximum is 3.
		 * @param {Number} index0 Zero-Based index in the first dimension.
		 * @param {Number} index1 Zero-Based index in the second dimension.
		 * @param {Number} index2 Zero-Based index in the third dimension.
		 * @returns {Promise<value, indexCount, index0, index1, index2>} A promise that is resolved or rejected.
		 */
		sendComplexValue: function(methodName, param, indexCount, index0, index1, index2) {
			return this._postMessage("SetComplexValue",{methodName: methodName, param:param, indexCount:indexCount, index0:index0, index1:index1, index2:index2}, true);
		},

		/**
		 * Internal Helper function to send messages from the element's iFrame to the webvisu.
		 */
		 _postMessage: function(messageType, data, awaitsResponse) {
			if (window["CdsInfo"]["TargetOrigin"] === undefined) {
				throw new TypeError("TargetOrigin is not yet set.");
			}
			var promiseId, promise;
			if (awaitsResponse) {
				promiseId = iPromiseId;
				promise = new Promise(function (resolve, reject) {
				promiseDict[iPromiseId] = new PromiseCallbacks(resolve, reject);
				});
				iPromiseId++;
			} else {
				promiseId = null;
				promise = Promise.resolve();
			}
			window.parent.postMessage({type:messageType, data: data, promiseId: promiseId, identification: window["CdsInfo"]["Identification"]}, window["CdsInfo"]["TargetOrigin"]);
			return promise;
		}
	};
}());

(function()
{
	/**
	 * Internal class to capsulate promises.
	 */
	PromiseCallbacks = function (resolve, reject){
		this.resolve = resolve;
		this.reject = reject;
	};

	PromiseCallbacks.prototype = {

		getResolveCallback: function() {
			return this.resolve;
		},

		getRejectCallback: function() {
			return this.reject;
		}
	};
}());

window["CDSWebVisuAccess"] = new CDSWebVisuAccess();
window["IecTypes"] = IecTypes;
window["TypeIds"] = TypeIds;